function value_s( score ) {
var pathArray = window.location.pathname.split('/');
	var newpath = '';
	if (pathArray[1] && pathArray[1] != 'arcade') {
		newpath = '/' + pathArray[1];
	}
		scorepost(window.location.protocol+'//' + window.location.hostname + newpath+'/index.php?act=Arcade&do=newscore', {
			  gname : 'SweetSweetjsnwulf71',
			  gscore: score
			  }); 
}